!Code for evolving initial conditions and saving the phase space of a selected map in subroutine
!The map is in the form (x0,y0) -> (x1,y1). Ex:
!       y1=y0+k*sin(x0)
!       x1=x0+y1
        program phase_space
        implicit real*8(a-h,o-z) !a-h and o-z are double, i,j,k,l,m,n are integer 
        
        parameter(nx=1d1) !Number of initial conditions in x
        parameter(ny=1d1) !Number of initial conditions in y
        parameter(iter=1d4) !Number of iteration
        parameter(lapis=1d1) !Auxiliary for write less points. Ex: Only save points each 100 iteration

        open(10,file='initial_conditions.dat',status='unknown') !File's name for its initial conditions
        open(20,file='phase_space.dat',status='unknown') !File's name for its phase space
        !OBS: Files at the same folder of the .for

        pi=dacos(-1.0d0) !Definition for pi
        pi2=2.0d0*pi !Defition for two pi

!Value of the control paramer of the selected map
        param=1.46 !Ex for the Standard map 
        !param=0.001 !Ex for the Fermi-Ulam map
        !param=0.6 !Ex for the Boozer map

!Size of the phase space. It can be set for specific region
        !Usual for the Standard map
        xspacemin=0
        xspacemax=pi2
        yspacemin=0
        yspacemax=pi2

        !Usual for the Fermi-Ulam map
        ! xspacemin=0.0
        ! xspacemax=pi2
        ! yspacemin=0.0
        ! yspacemax=2.5*dsqrt(param)
        
        !Usual for the Boozer map
        ! xspacemin=-0.0025
        ! xspacemax=0.0025
        ! yspacemin=0.996
        ! yspacemax=1

!Initial conditions distributed along all phase space
        x0min=xspacemin
        x0max=xspacemax
        deltax=(x0max-x0min)/nx
        y0min=yspacemin
        y0max=yspacemax
        deltay=(y0max-y0min)/ny
         
!Initial conditions centered in specific region       
        ! x0center=-5.16793e-05  
        ! y0center=0.997129
        ! icbox=1d-3
        ! x0min=x0center-icbox
        ! x0max=x0center+icbox
        ! deltax=(x0max-x0min)/nx
        ! y0min=y0center-icbox
        ! y0max=y0center+icbox
        ! deltay=(y0max-y0min)/ny
    
        do i=1,nx !Loop over inition conditions in x
            x0=x0min+i*deltax
            do j=1,ny !Loop over inition conditions in y
                y0=y0min+j*deltay
                auxx=x0
                auxy=y0
                write(10,*)x0,y0 !Write initial conditions in file
                do k=0,iter !Evolution of the selected map
                    call standard_map(param,auxx,auxy,x1,y1) !Selected map
                    if (x1 .ge. xspacemin) then
                        if (x1 .le. xspacemax)then
                            if (y1 .ge. yspacemin) then
                                if (y1 .le. yspacemax)then
                                    if (mod(k,lapis) .eq. 0) then
                                        write(20,*)x1,y1 !Write phase space in file
                                        kk=0
                                    endif 
                                end if
                            end if   
                        end if
                    end if   
                end do
            end do
        end do
        
        close(10)
        close(20)

        end 

!Chirikov-Taylor or Standard map
        subroutine standard_map(param,x0,y0,x1,y1)
            implicit real*8(a-h,o-z)
            pi=dacos(-1.0d0)
            pi2=2.0d0*pi
            y1=y0+param*dsin(x0)
            y1=dmod(y1,pi2)
            if(y1.lt.0.0d0) y1=y1+pi2
            x1=x0+y1+pi
            x1=dmod(x1,pi2)
            if(x1.lt.0.0d0) x1=x1+pi2
            x0=x1
            y0=y1
            return
        end subroutine

!Simplified Fermi-Ulam map
        subroutine fermi_ulam_map(param,x0,y0,x1,y1)
            implicit real*8(a-h,o-z)
            pi=dacos(-1.0d0)
            pi2=2.0d0*pi
            y1=dabs(y0-2*param*dsin(x0))
            x1=dmod(x0+2/y1,pi2)
            if(x1.lt.0.0d0) x1=x1+pi2
            x0=x1
            y0=y1
            return
        end subroutine	          

!Boozer map for the divertor tokamak
        subroutine boozer_map(param,x0,y0,x1,y1)
            implicit real*8(a-h,o-z)
            x1=x0-param*y0*(1-y0)
            y1=y0+param*x1
            x0=x1
            y0=y1
            return
        end subroutine 


!Author: Matheus Palmero